/* $Workfile: OsaLinuxCppRtl.h $   $Revision: 15 $    $Modtime: 11/01/04 3:36p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2003, All Rights Reserved

*******************************************************************************/

#ifndef OSA_LINUX_CPP_RTL_H
#define OSA_LINUX_CPP_RTL_H

#include "../OsaRtl.h"
#include "OsaLinuxC.h" 

/******************************************************************************
******************************************************************************/
class OsaAtomicInt {
public:
  inline operator OTX_UINT32() const volatile             { return (OTX_UINT32) atomic_read(&myval); }
  inline OTX_UINT32 operator= (const OTX_UINT32 ui32)     { atomic_set(&myval, (int) ui32); }
  inline OTX_UINT32 operator+=(const int i)               { atomic_add(i, &myval); };
  inline OTX_UINT32 operator++(const int)                 { int i = atomic_read(&myval); atomic_inc(&myval); return (OTX_UINT32) i; };
  inline OTX_UINT32 operator++()                          { atomic_inc(&myval); return atomic_read(&myval); };
  inline OTX_UINT32 operator--(const int)                 { int i = atomic_read(&myval); atomic_dec(&myval); return (OTX_UINT32) i; };
  inline OTX_UINT32 operator--()                          { atomic_dec(&myval); return atomic_read(&myval); };

protected:
  atomic_t myval;
};

/******************************************************************************
******************************************************************************/
class OsaSpinLockImpl {
public:
  OsaSpinLockImpl()
  {
    m_oaiLocked = 0;
    OsaSpinLockImpl_Ctor(&m_pSLock);
  }

  ~OsaSpinLockImpl()
  {
    if (IsLocked()) {
      Unlock();
    }
    OsaSpinLockImpl_Dtor(&m_pSLock);
  }

  void Lock()   
  {
    OsaSpinLockImpl_Lock(m_pSLock, &m_sqSavedFlags);
    m_oaiLocked++;
  }
  
  void Unlock()
  {
    m_oaiLocked--;
    OsaSpinLockImpl_Unlock(m_pSLock, m_sqSavedFlags);
  }

  bool IsLocked()
  {
    return (bool) (m_oaiLocked != 0);
  }

private:
  void          *m_pSLock;
  __u32         m_sqSavedFlags;
  OsaAtomicInt  m_oaiLocked;
};


/******************************************************************************
******************************************************************************/
class OsaDpcImpl {
public:
  OsaDpcImpl()
  {
    pDpcFct = (void(*)(void *, void *, void *, void *)) NULL;
    pDpcFctContext = (void *) NULL;
  }

  ~OsaDpcImpl()
  {
  }

  virtual void OsaDpcInitialize(void *pFct, void *pContext)
  {
    pDpcFct = (void(*)(void *, void *, void *, void *)) pFct;
    pDpcFctContext = pContext;
  }

  virtual bool OsaDpcInsertQueue(void *pArg1, void *pArg2)
  {
    if (pDpcFct) {
      osalinux_request_dpc(pDpcFct, NULL, pDpcFctContext, pArg1, pArg2);
      return true;
    } else {
      return false;
    }
  }

private:
  void (*pDpcFct) (void *pDpcContext, void *pContext, void *pArg1, void *pArg2);
  void *pDpcFctContext;
};


/******************************************************************************
******************************************************************************/
class OsaEventImpl {
public:
	  OsaEventImpl()
      : m_nEventSet(0)
    {
      OsaEventImpl_Ctor(&m_pWaitQHead);
    }

  ~OsaEventImpl()
  {
    // Release possible waiting threads before destroying
    // This is an arbitrary decision on my part /LME990923
    SetEvent();
    OsaEventImpl_Dtor(&m_pWaitQHead);
  }

  bool SetEvent()
  {
    int nEventSetBefore = m_nEventSet; 
    m_nEventSet = 1;
    OsaEventImpl_wake_up(&m_pWaitQHead);
    return nEventSetBefore ? true : false;
  }

  void ClearEvent()
  {
    m_nEventSet = 0;
  }

private:
  friend int OsaWaitForSingleEvent(OsaEventImpl *pEvent, OsaUint32 nWaitMaxMs);

  void              *m_pWaitQHead;  
  OsaUint32         m_nEventSet;
  OsaSpinLockImpl   m_access;
};


/******************************************************************************
  A Timeout of zero allows the testing of a set of wait conditions and for 
  conditionally performing any side effects if the wait can be immediately
  satisfied
******************************************************************************/
int OsaWaitForSingleEvent(OsaEventImpl *pEvent, OsaUint32 nWaitMaxMs);

/******************************************************************************
******************************************************************************/
class OsaSemaphoreImpl {
public:
  OsaSemaphoreImpl()
  {
    OsaSemaphoreImpl_Ctor(&m_Semaphore);
  }

  ~OsaSemaphoreImpl()
  {
    OsaSemaphoreImpl_Dtor(&m_Semaphore);
  }

  void Release()
  {
    OsaSemaphoreImpl_Release(&m_Semaphore);
  }

private:
  friend int OsaWaitForSemaphore(OsaSemaphoreImpl *pSemaphore);
  void *m_Semaphore;
};

int OsaWaitForSemaphore(OsaSemaphoreImpl *pSemaphore, OsaUint32 nWaitMaxMs);

#endif
